from __future__ import absolute_import, unicode_literals

import logging
import os
import traceback
from urllib.parse import quote

import requests
from celery import shared_task
from django.conf import settings
from django.shortcuts import reverse



import logging
import traceback
from threading import Thread
from typing import Optional

from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import render_to_string
from django.utils.translation import gettext as _

from .utils import import_class

logger = logging.getLogger("notifier")


class Emailer:
    def __init__(
        self,
        subject: str,
        receivers: list,
        template: Optional[str],
        context: dict,
        smtp_account: str='default',
        final_message: str = None,
        files: list = None,
        **kwargs,
    ):
        from magic_notifier.settings import NOTIFIER_SMTP
        self.email_settings:dict = NOTIFIER_SMTP[smtp_account]
        self.email_client = import_class(self.email_settings["CLIENT"])

        self.connection = self.email_client.get_connection(self.email_settings)
        self.subject: str = subject
        self.receivers: list = receivers
        self.template: Optional[str] = template
        self.context: dict = context if context else {}
        self.final_message = final_message
        self.threaded: bool = kwargs.get("threaded", False)
        self.files: list = files

    def send(self):
        if self.threaded:
            t = Thread(target=self._send)
            t.setDaemon(True)
            t.start()
        else:
            self._send()

    def _send(self):
        try:
            logger.info(f"sending emails to {self.receivers}")
            for user in self.receivers:
                # activate(user.settings.lang)
                if isinstance(user, str):
                    user = User(email=user, username=user)

                ctx = self.context.copy()
                ctx["user"] = user
                logger.info(f" Sending to user {user.username} with context {ctx}")

                if self.template:
                    try:
                        html_content = render_to_string(
                            f"notifier/{self.template}/email.html", ctx
                        )  # render with dynamic value
                        logger.debug(html_content)
                    except TemplateDoesNotExist:
                        html_content = None

                    text_content = render_to_string(
                    f"notifier/{self.template}/email.txt", ctx
                    )  # render with dynamic value
                    logger.debug(text_content)
                else:
                    html_content = text_content = self.final_message

                msg = EmailMultiAlternatives(
                    self.subject,
                    text_content,
                    self.email_settings["FROM"],
                    [user.email],
                    connection=self.connection,
                )
                if html_content:
                    msg.attach_alternative(html_content, "text/html")

                if self.files:
                    for i, pos_file in enumerate(self.files):
                        if isinstance(pos_file, str):
                            msg.attach_file(pos_file)
                        elif isinstance(pos_file, tuple):
                            name, f = pos_file
                            if hasattr(f, 'read'):
                                msg.attach(name, f.read())
                            else:
                                logging.warning(f"file {name} can't be added to mail because it is not a file-like object")
                        elif hasattr(pos_file, 'read'):
                            msg.attach(f"file {i+1}", pos_file.read())
                        else:
                            logging.warning(f"discarding possible file {pos_file}")
                msg.send()
        except:
            logger.error(traceback.format_exc())
            print(traceback.format_exc())


if __name__ == "__main__":
    emailer = Emailer("Reset code", ["jefcolbi@gmail.com"], "reset", {}, "account")
    emailer.send()



# Generated by Django 3.2.4 on 2021-08-22 13:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('magic_notifier', '0003_auto_20210819_1451'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='id',
            field=models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



import logging
import traceback
from threading import Thread

from django.template.loader import render_to_string

try:
    from asgiref.sync import async_to_sync
    from channels.layers import get_channel_layer
except:
    pass
import json

from django.template import Context, Template

from magic_notifier.utils import NotificationBuilder

logger = logging.getLogger("notif")


class Pusher:
    def __init__(
        self, subject, receivers: list, template: str, context: dict, **kwargs
    ):
        self.receivers: list = receivers
        self.template: str = template
        self.context: dict = context
        self.threaded: bool = kwargs.get("threaded", False)
        self.image = kwargs.get("image")

    def send(self):
        if self.threaded:
            t = Thread(target=self._send)
            t.setDaemon(True)
            t.start()
        else:
            self._send()

    def _send(self):
        try:
            for user in self.receivers:
                ctx = self.context.copy()
                ctx["user"] = user
                data_str = json.dumps(ctx.get("data", {}), indent=4)
                ctx["data"] = data_str
                push_content = render_to_string(
                    "{}/push.json".format(self.template), ctx
                )

                print(push_content)

                event: dict = json.loads(push_content)
                print(event)

                not_builder = (
                    NotificationBuilder(event["text"])
                    .type(event["type"], event["sub_type"])
                    .link(event["link"])
                    .mode(event["mode"])
                    .data(event["data"])
                    .actions(event["actions"])
                    .user(user)
                )

                if self.image:
                    not_builder.image(self.image)

                not_builder.save()

                # return

                push_channel = user.settings.push_channel
                print(f"user is waitin to channel {push_channel}")

                channel_layer = get_channel_layer()
                async_to_sync(channel_layer.send)(push_channel, event)
        except:
            logger.error(traceback.format_exc())


if __name__ == "__main__":
    pass



import binascii
import ctypes
import json
import logging
import traceback
from datetime import date, datetime, timedelta
from pathlib import Path

from channels.db import database_sync_to_async
from channels.generic.websocket import WebsocketConsumer
from django.core.exceptions import ObjectDoesNotExist
from django.utils import timezone
from notif.models import Notification
from notif.models_serializers import NotificationSerializer
from rest_framework.authtoken.models import Token

logger = logging.getLogger("notif")


class PushNotifConsumer(WebsocketConsumer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.token: str = None
        self.user = None

    def connect(self):
        try:
            self.accept()

            self.token = self.scope["url_route"]["kwargs"]["token"]
            db_tok = Token.objects.get(key=self.token)
            self.user = db_tok.user

            self.user.settings.push_channel = self.channel_name
            self.user.settings.save()

            logger.info("Accepted")
        except Exception as e:
            print(traceback.print_exc())
            logging.error(traceback.format_exc())

    def disconnect(self, close_code):
        try:
            self.user.settings.push_channel = None
            self.user.settings.save()
        except Exception as e:
            print(traceback.format_exc())
            logging.error(traceback.format_exc())

    def receive(self, text_data):
        event = json.loads(text_data)
        logger.info("{} >> {}".format(self.user, text_data))

        event_handler = getattr(self, event["type"].lower().replace(".", "_"), None)
        if callable(event_handler):
            event_handler(event)

    def notify(self, data: dict):
        self.send(json.dumps(data))

    def notification(self, data: dict):
        self.send(json.dumps(data))

    def unread(self, event: dict):
        notifs = Notification.objects.filter(user=self.user, read__isnull=True)
        event["count"] = len(notifs)
        event["notifications"] = NotificationSerializer(notifs, many=True).data
        self.send(json.dumps(event))

    def markread(self, event: dict):
        notifs = Notification.objects.filter(user=self.user, read__isnull=True)
        notifs.update(read=timezone.now())
        event["success"] = True
        self.send(json.dumps(event))



from django.conf import settings

from tests.example.settings import NOTIFIER

AVAILABLE_MODES = [("user", "User"), ("admin", "Admin")]
NOTIFIER_SETTINGS = getattr(settings, "NOTIFIER", {})

SMTP_ACTIVE = NOTIFIER_SETTINGS.get('SMTP_ACTIVE', True)
SMS_ACTIVE = NOTIFIER_SETTINGS.get('SMS_ACTIVE', False)
PUSH_ACTIVE = NOTIFIER_SETTINGS.get('PUSH_ACTIVE', False)

NOTIFIER_AVAILABLE_MODES = NOTIFIER_SETTINGS.get('DEFAULT_MODES', AVAILABLE_MODES)
NOTIFIER_DEFAULT_MODE = NOTIFIER_SETTINGS.get("DEFAULT_MODE", "user")

NOTIFIER_THREADED = NOTIFIER_SETTINGS.get('THREADED', False)

if SMTP_ACTIVE:
    NOTIFIER_SMTP = NOTIFIER_SETTINGS.get('SMTP', {})

    if "default" not in NOTIFIER_SMTP:
        # we build the default dict from django standard smtp settings
        assert settings.EMAIL_HOST, "You have not defined any DEFAULT SMTP settings and no django smtp settings detected."
        NOTIFIER_SMTP["default"] = {
            "HOST": settings.EMAIL_HOST,
            "PORT": settings.EMAIL_PORT,
            "USER": settings.EMAIL_HOST_USER,
            "FROM": settings.DEFAULT_FROM_EMAIL,
            "PASSWORD": settings.EMAIL_HOST_PASSWORD,
            "USE_SSL": getattr(settings, "EMAIL_USE_SSL", False),
            "USE_TLS": getattr(settings, "EMAIL_USE_TLS", False),
            "CLIENT": "magic_notifier.email_clients.django_email.DjangoEmailClient"
        }

NOTIFIER_SMS = NOTIFIER_SETTINGS.get('SMS', {})
NOTIFIER_SMS_DEFAULT_GATEWAY = NOTIFIER_SETTINGS.get('DEFAULT_GATEWAY', 'default')
NOTIFIER_SMS_GATEWAYS = NOTIFIER_SETTINGS.get('GATEWAYS', {})



from rest_framework.serializers import ModelSerializer

from magic_notifier.models import Notification


class NotificationSerializer(ModelSerializer):
    class Meta:
        model = Notification
        fields = (
            "text",
            "type",
            "sub_type",
            "link",
            "mode",
            "image",
            "actions",
            "data",
            "read",
            "sent",
        )



"""
"""


class BaseSmsClient:

    @classmethod
    def send(cls, number: str, text: str):
        raise NotImplementedError

if __name__ == "__main__":
    CGSmsClient.send("0000000000", "Cava ma tigresse")



import importlib
import logging
import traceback
from threading import Thread
from typing import Optional

from django.template.loader import render_to_string

from .settings import NOTIFIER_SMS_DEFAULT_GATEWAY, NOTIFIER_SMS_GATEWAYS

logger = logging.getLogger("notifier")


class ExternalSMS:

    def __init__(self, receivers: list, context: dict, template: Optional[str]=None,
        final_message: Optional[str]=None, sms_gateway: Optional[str]=None, **kwargs):
        self.receivers: list = receivers
        self.template: Optional[str] = template
        self.context: dict = context
        self.threaded: bool = kwargs.get("threaded", False)
        self.final_message: Optional[str] = final_message
        self.sms_gateway = NOTIFIER_SMS_DEFAULT_GATEWAY if sms_gateway is None else sms_gateway

        NOTIFIER_SMS_GATEWAY = NOTIFIER_SMS_GATEWAYS[self.sms_gateway]
        NOTIFIER_SMS_CLIENT = NOTIFIER_SMS_GATEWAY["CLIENT"]

        module_name, class_name = NOTIFIER_SMS_CLIENT.rsplit(".", 1)
        module = importlib.import_module(module_name)
        assert hasattr(module, class_name), "class {} is not in {}".format(class_name, module_name)
        self.client_class = getattr(module, class_name)

    def send(self):
        if self.threaded:
            t = Thread(target=self._send)
            t.setDaemon(True)
            t.start()
        else:
            self._send()

    def _send(self):
        try:
            for rec in self.receivers:
                ctx = self.context.copy()
                ctx["user"] = rec
                if self.final_message:
                    sms_content = self.final_message
                else:
                    sms_content = render_to_string("{}/sms.txt".format(self.template), ctx)

                self.client_class.send(rec.number, sms_content)
        except:
            logger.error(traceback.format_exc())


if __name__ == "__main__":
    pass



import logging

from django.conf import settings
from twilio.rest import Client

from .base import BaseSmsClient

logger = logging.getLogger("notifier")


class TwilioClient(BaseSmsClient):

    @classmethod
    def send(cls, number: str, text: str):
        account = settings.NOTIFIER["TWILIO"]["ACCOUNT"]
        token = settings.NOTIFIER["TWILIO"]["TOKEN"]
        from_number = settings.NOTIFIER["TWILIO"]["FROM_NUMBER"]
        client = Client(account, token)
        res = client.messages.create(from_=from_number, to=number, body=text)
        return res



import logging
import traceback
from typing import Optional, Union

from django.contrib.auth import get_user_model
from django.db import models

from magic_notifier.emailer import Emailer
from magic_notifier.pusher import Pusher
from magic_notifier.settings import NOTIFIER_THREADED
from magic_notifier.smser import ExternalSMS

User = get_user_model()

logger = logging.getLogger("notifier")


def notify(
    vias: list,
    subject: str = None,
    receivers: Union[str, list, models.QuerySet, models.Manager] = None,
    template: str = None,
    context: dict = {},
    final_message: str = None,
    smtp_account: str = 'default',
    sms_gateway: Optional[str] = None,
    files: list = None,
    threaded: bool = None,
):
    """Send a notification

    :param vias: list of sendin method. can be email|sms|push
    :param subject: the subject of the notification
    :param receivers: the receivers list. can be a list of user instances or a string representing users like admins|staff|all|all-staff|all-admins
    :param template: the email template. ignored if final message is passed
    :param context: the context
    :param final_message: the direct content of the notification
    :param smtp_account: the smtp account to use
    :param threaded: if the notification must be send via a worker thread
    :return:
    """
    logger.debug(f"Sending {subject} to {receivers} via {vias}")
    threaded = threaded if threaded is not None else NOTIFIER_THREADED

    assert subject, "subject not defined"


    if isinstance(receivers, str) and receivers in ["admins", "staff", "all", "all-staff", "all-admins"]:
        if receivers == "admins":
            receivers = User.objects.filter(is_superuser=True)
        elif receivers == "staff":
            receivers = User.objects.filter(is_staff=True)
        elif receivers == "all":
            receivers = User.objects.all()
        elif receivers == "all-staff":
            receivers = User.objects.exclude(is_staff=True)
        elif receivers == "all-admins":
            receivers = User.objects.exclude(is_superuser=True)
        else:
            raise ValueError(f"'{receivers}' is not an allowed value for receivers arguments")

    assert isinstance(receivers, (list, models.Manager, models.QuerySet)), f"receivers must be a list at this point not {receivers}"

    for via in vias:
        try:
            if via == "email":
                em = Emailer(
                    subject,
                    receivers,
                    template,
                    context,
                    smtp_account,
                    threaded=threaded,
                    final_message=final_message,
                    files=files
                )
                em.send()

            elif via == "sms":
                ex_sms = ExternalSMS(receivers,context, threaded=threaded,
                    template=template, final_message=final_message,
                    sms_gateway=sms_gateway)
                ex_sms.send()

            elif via == "push":
                assert template, "template variable can't be None or empty"

                pusher = Pusher(
                    subject, receivers, template, context, threaded=threaded
                )
                pusher.send()

            else:
                logger.error(f"Unknown sending method {via}")

        except:
            logger.error(traceback.format_exc())



import importlib
import logging
import traceback

from django.contrib.auth import get_user_model

from magic_notifier.models import Notification

logger = logging.getLogger('notifier')

User = get_user_model()


class NotificationBuilder:
    def __init__(self, text):
        self.__text = text
        self.__user = None
        self.__type = None
        self.__sub_type = None
        self.__mode = None
        self.__actions = []
        self.__link = None
        self.__data = {}
        self.__image = None

    def text(self, text=None):
        if text is None:
            return self.__text

        if not isinstance(text, str):
            raise ValueError("text should be a string")

        self.__text = text
        return self

    def link(self, link=None):
        if link is None:
            return self.__link

        if not isinstance(link, str):
            raise ValueError("link should be a string")

        self.__link = link
        return self

    def mode(self, mode=None):
        if mode is None:
            return self.__mode

        if not isinstance(mode, str):
            raise ValueError("mode should be a string")

        self.__mode = mode
        return self

    def type(self, type: str = None, sub_stype: str = None):
        if type is None:
            return self.__type, self.__sub_type

        self.__type = type
        self.__sub_type = sub_stype
        return self

    def action(self, text=None, method: str = None, url: str = None, fields: dict = {}):
        if method is None or url is None or text is None:
            return self.__actions

        action = {"method": method, "url": url, "fields": fields, "text": text}
        self.__actions.append(action)
        return self

    def actions(self, actions: list = None):
        if actions is None:
            return self.__actions

        if not isinstance(actions, list):
            raise ValueError("actions should be a list")

        self.__actions.extend(actions)
        return self

    def user(self, user: User = None):
        if not user:
            return self.__user

        if isinstance(user, User):
            self.__user = user
        else:
            raise ValueError("Sender should be User or Reach instance")

        return self

    def data(self, data: dict = None):
        if data is None:
            return self.__data

        if isinstance(data, dict):
            self.__data.update(data)
            return self

        raise ValueError("data should be a dict")

    def image(self, image=None):
        if image is None:
            return self.__image

        self.__image = image
        return self

    def save(self):
        notif = Notification.objects.create(
            text=self.__text,
            link=self.__link,
            user=self.__user,
            data=self.__data,
            actions=self.__actions,
            image=self.__image,
            type=self.__type,
            sub_type=self.__sub_type,
        )
        return notif

    def show(self):
        return (
            f"(text={self.__text}, link={self.__link}, user={self.__user}, "
            f"type={self.__type}, sub_stype={self.__sub_type}, mode={self.__mode},"
            f"data={self.__data}, actions={self.__actions}, image={self.__image}"
        )


def import_class(class_path:str):
    module_name, class_name = class_path.rsplit(".", 1)
    module = importlib.import_module(module_name)
    assert hasattr(module, class_name), "class {} is not in {}".format(class_name, module_name)
    logger.debug('reading class {} from module {}'.format(class_name, module_name))
    cls = getattr(module, class_name)
    return cls


# Generated by Django 2.2.17 on 2021-08-19 14:51

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('magic_notifier', '0002_alter_notification_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='magic_notifications', to=settings.AUTH_USER_MODEL),
        ),
    ]
